/*
	Title:	Converter tool for changing valid sudoku grids into graph colouring problems
	file:	converter.cpp
	Does:	takes as progam arguments an inputfile (must be valid), an output file name
			and a 0 or 1 asking if we'd like screen output or not
	Source: Rhydian Lewis's Homepage at Prifysgol Caerdydd/Cardidd University
	Author:	Rhyd Lewis (December 2006) 
	email:	LewisR9@cf.ac.uk

	Compilation: Ansi C++ Code

	Inputs: input file name
			output file name
			(0/1) option for screen output (if using a shell script or batch then
					turning this off will be useful)

	Outputs: the generated graph in DIMACS standard format

	Copyright: Rhydian Lewis. Permission is granted to
		use and distribute freely for non-commercial purposes
		with appropriate attribution.
	
	Warranty: There is no warranty. Feel free to update this code yourself...
	
	Special Instructions: If any updates are made that improve this code then please let me know

	Cheers and Iachi Da!!!
	Rhyd Lewis 15/12/06
*/


#include "converter.h"
#include "extrafunctions.h"
#include "converterfunctions.h"

int main (int argc, char *argv[])
{
	if(argc != 4){
		cout<<"USAGE inputfile outputfile screenOutput(0/1)\n";
		exit(1);
	}
	
	
	//PROGRAM INTEGER VARIABLES
	int i, r, c;

	//READ IN THE PROGRAM ARGUMENTS
	ifstream inStream;
	inStream.open(argv[1]);
	if(inStream.fail()) {
		cout << "Error opening input file\n";
		exit(1);
	}
	ofstream outStream;
	outStream.open(argv[2]);
	if(outStream.fail()) {
		cout << "Error opening output file\n";
		exit(1);
	}

	bool doScreenOutput;
	if(atoi(argv[3])==1) doScreenOutput = true;
	else doScreenOutput = false;


	//MAKE WELCOME MESSAGE
	if(doScreenOutput){
		cout<<"\nRHYD LEWIS'S SUDOKU -> GRAPH COLOURING CONVERTER. WELCOME!\n";
		cout<<"----------------------------------------------------------\n\n";
	}
		


	
	//----------------------------------------------------------------------
	//NOW READ IN THE  SUDOKU GRID AND PUT INTO THE 2D ARRAY CALLED PUZZLE
	int order;
	inStream>>order;
	int numCells = order*order*order*order;
	int numRows = order*order;
	int numCols = order*order;

	int **puzzle = new int* [numRows];
	for(i=0; i<numRows; i++) puzzle[i] = new int [numCols];

	for(r=0; r<numRows; r++)for(c=0; c<numCols; c++)inStream>>puzzle[r][c];
	inStream.close();

	//NOW CHECK THE GRID TO SEE IF ITS VALID
	if(doScreenOutput) printGrid(puzzle,order,numCols);

	if(!gridOK(puzzle,order,numCols)){
		cout<<"Exiting...\n";
		exit(1);
	}



	//----------------------------------------------------------------------
	//IF WE ARE HERE THEN WE HAVE FOUND NO ERRORS WITH THE GRID. WE
	//CAN NOW COVERT THIS TO A GRAPH COLOURING PROBLEM.

	//First, set up the adjacency matrix (a 2D bool array)
	int numNodes = numCells;
	bool **adjacent = new bool* [numNodes];
	for(i=0; i<numNodes; i++) adjacent[i] = new bool [numNodes];

	//Now convert by filling in the adjacency matrix
	int edgesDueToFilledCells = 0;
	convertToGCol(puzzle,order,numCols,adjacent,numNodes,edgesDueToFilledCells);
	int totalNumEdges = calcNumEdges(adjacent, numNodes);

	//Now make the output file
	makeOutput(argv[2], outStream, numNodes, puzzle, adjacent, totalNumEdges, argv[1], order, edgesDueToFilledCells);

	//Print out a goodbye message and end
	if(doScreenOutput){
		cout<<"\nGraph Finished....\nNumber of Nodes = "<<numNodes<<"\nNumber of Edges = "<<totalNumEdges<<"\nEdges due to filled cells = "<<edgesDueToFilledCells<<endl;
		cout<<"\nCheck output file "<<argv[2]<<endl<<endl;
	}

	return 0;
}




